%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS syntax1.ql,v 1.7 1993/12/10 02:23:17 marku Exp


?- op(990,xfy,(-->)).


/*
** Is name an atom?
*/

check_atom(Name) :-
        check_or_writelist(
                atom(Name),
                [Name, ' is not an atom.', nl]).

check_atomic(Name) :-
        check_or_writelist(
                atomic(Name),
                [Name, ' is not atomic.', nl]).
/*
** Check syntax of a term in the current directory
*/

check_term(Term) :- 
	simplify_term(Term, SimpTerm),
	substitute(SimpTerm, Sub, SubTerm),
	check_subterm(SubTerm),
	check_sub(Sub).

check_subterm(Term) :- 
	var(Term), !.

check_subterm(Term) :- 
	is_object_var(Term), !.

check_subterm(Term) :- 
	quantify(Term, Q, _, Body), !,
	check_or_writelist(
		inherited_symbol(Q, quant, _),
	        ['Quantifier ', Q, ' not defined in this theory', nl]),
	check_term(Body).


check_subterm(Term) :- 
	Term =.. [Func|Args], !,
	length(Args, N),
	check_or_writelist(
		inherited_symbol(Func, N, _), 
		['Function ', (Func/N), ' not defined in this theory', nl]),
	check_terms(Args).


check_sub(Sub) :-
	\+     (member(H, Sub),
		member((R/_), H),
		\+  check_term(R)).


check_terms(T) :-
	\+     (member(Term, T),
		\+  check_term(Term)).


/*
** Opening rule syntax
*/

check_ruleterm([]) :- !.
check_ruleterm([Term|List]) :-
        Term = (Term1 --> Term2),
        check_term(Term1),
	check_term(Term2),
	check_ruleterm(List).

check_ruleequ([]) :- !.
check_ruleequ([(Eq1 --> Eq2)|List]) :-
	once( (atom(Eq1), Eq1 = E1  ;  Eq1 = rev(E1), atom(E1)) ),
	once( (atom(Eq2), Eq2 = E2  ;  Eq2 = rev(E2), atom(E2)) ),
	get_up(preorder, [E1]),
	get_up(preorder, [E2]),
	check_ruleequ(List).


