%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS io.ql,v 1.10 1993/12/10 02:23:32 marku Exp


/*
** Get command from user or macro file
*/

get_command(Prompt, C) :- 
	(\+ in_macro ->
	    writelist(Prompt)
	;
	    true
	), 
	readR(C1), !,
	get_command(Prompt, C1, C).

get_command(Prompt, C) :- 
	get_command(Prompt, C).

get_command(Prompt, A, C) :-
	A == end_of_file,
	in_macro, !,
	exit_macro,
	get_command(Prompt, C).

get_command(_, A, bye) :-
	A == end_of_file, !, nl.

get_command(_, C, C).


/*
** Ask the user a question, default to affirmative
*/

question(Question) :-
        writelist([Question, ' (yes) : ']),
        getl([Char|_]),  
	\+  member(Char, "Nn").

/*
** Multiple write's
**
**  The atom 'nl' causes a newline to be written.
**  The atom 'tab' causes a literal '\t' to be written.
**  The term 'tab(N)' causes N spaces to be written.
*/

writelist(L) :-
	writelist_indent(L, []).


/*
 * writelist_indent(+List, +Indent).
 *
 *  Writes the elements of List, but treats 'nl' atoms
 *  in List as meaning '\n' followed by Indent.
 *  Indent can be an atom or a list of terms to be written by writelist/1.
 */

writelist_indent([],_) :-
	!.

writelist_indent([T|Ls], Indent) :-
	atom(T),
	T == nl,
	!,
	nl,
	writelist(Indent),
	writelist_indent(Ls, Indent).

writelist_indent([T|Ls], Indent) :-
	atom(T),
	T == tab,
	!,
	put(9),
	writelist_indent(Ls, Indent).

writelist_indent([T|Ls], Indent) :-
	functor(T,tab,1),
	arg(1,T,N),
	integer(N),
	!,
	tab(N),
	writelist_indent(Ls, Indent).

writelist_indent([T|Ls], Indent) :-
	!,
	writeR(T,_),
	writelist_indent(Ls, Indent).

writelist_indent(T, _) :-
	writeR(T,_).


/*
 * spaces(+N, -Atom).
 *
 *  creates Atom, containing N spaces.
 */
spaces(N, Atom) :-
	spaces_list(N, List),
	atom_chars(Atom,List).

spaces_list(0, []).
spaces_list(N, [32|S]) :-
	N > 0,
	N1 is N - 1,
	spaces_list(N1,S).


check_or_writelist(Code, _) :- once(Code), !.
check_or_writelist(_, List) :- writelist(List), !, fail.


/*
** goes in unif.ql eventually
*/

nfi :- write_delayed_problems.
