%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS newvars.ql,v 1.10 1993/12/10 02:23:46 marku Exp


/*
** Neato full-unify replacement, with automated syntax checking
*/


thawing_new_vars(Code) :-
	ipLookup(newvar_flag, true), !,
	ipLookup(newvars, Old), 
	ipSet(newvar_flag, fail),
	ipSet(newvars, []),
	'$thaw2'(Old),

        Code,

	ipLookup(newvars, L),
	nv_collect_variables_in_list(L, [], L1, [], F1),
	nv_collect_variables_in_list(Old, L1, XL, F1, F2),
	check_symbols(F2),
	'$freeze2'(XL), 
        ipSet(newvars, XL),
	ipSet(newvar_flag, true).

thawing_new_vars(Code) :-
        ipLookup(newvars, Old),
        ipSet(newvars, []),

        Code,

        ipLookup(newvars, L),

	nv_collect_variables_in_list(L, [], L1, [], F1),
	nv_collect_variables_in_list(Old, L1, XL, F1, F2),
	check_symbols(F2),

        ipSet(newvars, XL).



check_symbols(L) :-
	\+ (member((A, B), L),
	    \+ inherited_symbol(A, B, _)).


%%  nv_collect_variables_in_list(+List, +VIn, ?VOut, +FIn, ?FOut)
%%
%%
%%  Collect the thawed variables and (Function/Arity) pairs in List

nv_collect_variables_in_list([], VIn, VIn, FIn, FIn) :- !.
nv_collect_variables_in_list(List, VIn, VOut, FIn, FOut) :-
    setFlag(localfix, on, Old),
    nv_collect_vars_in_list(List, VIn, VOut, FIn, FOut),
    setFlag(localfix, Old, _), !.


nv_collect_vars_in_list([], VIn, VIn, FIn, FIn) :- !.
nv_collect_vars_in_list([Term|Terms], VIn, VOut, FIn, FOut) :-
    nv_collect_vars(Term, VIn, V1, FIn, F1),
    nv_collect_vars_in_list(Terms, V1, VOut, F1, FOut), !.



nv_collect_vars(Term, VIn, VOut, FIn, FOut) :-
    substitute(Term, Subs, T),
    nv_collect_subs(Subs, VIn, V1, FIn, F1),
    nv_collect_var(T, V1, VOut, F1, FOut).

nv_collect_var(Term, VIn, VIn, FIn, FIn) :-
    '$is_local_object_var'(Term), !.


nv_collect_var(Term, VIn, VOut, FIn, FOut) :-
    functor(Term, F, N), !,
    Term =.. [F|Args],
    '$add_to_vars'((F, N), FIn, F1), 
    nv_collect_vars_in_list(Args, VIn, VOut, F1, FOut).

nv_collect_var(Term, VIn, VOut, FIn, FOut) :-
    quantify(Term, Q, X, T), !,
    '$add_to_vars'((Q, quant), FIn, F1), 
    nv_collect_vars_in_list([X, T], VIn, VOut, F1, FOut).

nv_collect_var(Term, VIn, VIn, FIn, FIn) :-
    '$frozen'(Term), !. 

nv_collect_var(Term, VIn, VOut, FIn, FIn) :-
    '$add_to_vars'(Term, VIn, VOut).

nv_collect_subs([], VIn, VIn, FIn, FIn) :- !.
nv_collect_subs([Sub|Subs], VIn, VOut, FIn, FOut) :-
    nv_collect_sub(Sub, VIn, V1, FIn, F1),
    nv_collect_subs(Subs, V1, VOut, F1, FOut).

nv_collect_sub([], VIn, VIn, FIn, FIn) :- !.
nv_collect_sub([T/X|Sub], VIn, VOut, FIn, FOut) :-
    nv_collect_vars(X, VIn, V1, FIn, F1),
    nv_collect_vars(T, V1, V2, F1, F2),
    nv_collect_sub(Sub, V2, VOut, F2, FOut).



freeze_new_vars_in(Term, I, L) :-
	'$collect_variables'(Term, var, [], VarList),
	thawed_sublist(VarList, I, L),
	freeze(L).

thawed_sublist(A, B, C) :- thawed_sublist(A, B, C, _).

thawed_sublist([], I, I, []).
thawed_sublist([A|B], I, C, [A|D]) :- '$frozen'(A),!,thawed_sublist(B, I, C, D).
thawed_sublist([A|B], I, [A|C], D) :- thawed_sublist(B, I, C, D).


/*
** Additional commands
*/

freezing_vars(New, Code) :-
        freeze_new_vars_in(New, [], L1),
        once(Code),
        ipLookup(newvars, L),
        append(L1, L, L2),
        ipSet(newvars, L2).

inst_var(Term1, Term2) :- 
	demote_var(Term1),
	thawing_new_vars(once((Term1 = Term2, incomplete_retry_delays))).

match_terms(Term1, Term2) :-
	thawing_new_vars(once((Term1 = Term2, incomplete_retry_delays))).

/*
** assimilate_new_vars(+Term).
**
**  Freeze all (object and meta) variables in Term that are not already
**  frozen and add them to the 'newvars' list (an implicit parameter).
*/

assimilate_new_vars([]).
assimilate_new_vars([V|Vars]) :- 
	ipLookup(newvars, L),
	'$collect_variables'([V|Vars], var, [], VarList),
        thawed_sublist(VarList, L, NewL),
	(ipLookup(newvar_flag, true) -> freeze(NewL) ; true),
	ipSet(newvars, NewL).

%% promote_var(+Var)
%%
%% Will add all members of Var to proof-vars.  Removes from new-vars those 
%% members of Var which appear in it

promote_var(Var) :- 
	'$collect_variables'(Var, var, [], VarList),
	ipLookup(proofvars, PL),
	thawing_new_vars(
		(thawed_sublist(VarList, PL, NewPL),
		 '$freeze2'(NewPL))),
	ipSet(proofvars, NewPL).

%% demote_var(+Var)
%%
%% Will move those variables in Var which are in the proof-var list
%% to the new-var list.

demote_var(Var) :- 
        ipLookup(newvars, NL),
        ipLookup(proofvars, PL),
        '$collect_variables'(Var, var, [], VarList),

	\+ '$check_delayed_problem'('='),    %% from 'thaw' in qp source
	'$thaw2'(VarList),

	thawed_sublist(PL, NL, NewNL, NewPL),
        (ipLookup(newvar_flag, true) -> freeze(NewNL) ; true),
        ipSet(newvars, NewNL),
	ipSet(proofvars, NewPL).


is_newvarlist(NV)         :- ipLookup(newvars, NV).
is_proofvarlist(PV)       :- ipLookup(proofvars, PV).




