%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS browse.ql,v 1.9 1993/12/10 02:18:33 marku Exp


:- declare(browse/1).
:- declare(browse/2).


browse(Thm) :- browse(Thm, _).

browse(Thm, Form) :-
        decode_rule(Thm, CD, Name),
        atom(Name),
	(var(CD) -> current_dir(CD) ; true),
        get_proof_record(CD:Name, proof_of(_, Form, _, Proof)),
        reportProof(Proof).

reportProof(P) :-
	reportProof(P, []).

reportProof(P, W) :-
	system(clear),
	\+ \+ reportProof('', P, W, true, []),
	get_command(['Open Wrappers: ', W, ': '], NW), 
	once(reportProof(P, W, NW)).



reportProof(_, _, quit).
reportProof(_, _, bye).
reportProof(P, W, E) :- integer(E), reportProof(P, [E | W]).
reportProof(P, W, + E) :- integer(E), development, !, reportProof(P, [+(E) | W]).
reportProof(P, W, - E) :- !, delete(E, W, NW), reportProof(P, NW).
reportProof(P, W, []) :- !, reportProof(P, []).
reportProof(P, W, [A|B]) :- !, reportProof(P, [A|B]).
reportProof(P, W, _) :-
        get_command(['Open Wrappers: ', W, ': '], NW), 
        once(reportProof(P, W, NW)).


reportProof(I, (Equ, FL, Hyps), W, Pr, Hout) :-
	(Pr -> bar(I) ; true),
	   (Pr -> atom_append('|  ', I, J) ; J = I),
           reportFoci(J, FL, W, Pr, All),
	   partition_hyps(All, Used, _),
           reportHyps(J, Hyps, Used, W, Pr, Hout),
	(Pr -> bar(I) ; true).


write_wrapper((A, B)) :- !, ((B = []) -> true ; write_elems(B)).
write_wrapper(B)      :-    ((B = []) -> true ; write_elems(B)).



reportWrap(I, F, WN, Term, FL, W, Pr, H) :-
	check_open(WN, W, Pr2),
	reportWrap(I, F, WN, Term, FL, Pr, W, Pr2, H).

reportWrap(I, F, WN, Term, FL, P, W2, P, H) :-
	append(FL2, [_], FL),
	reportFoci(I, FL2, W2, P, H).

reportWrap(I, F, + N, Term, FL, true, W2, fail, H) :-
	member(N, W2), !,
	write(I), write_wrapper(Term), nl,
	writeln(I),
        reportFoci(I, FL, W2, fail, H),
	write(I), writeR(F, _), nl.

reportWrap(I, F, + N, Term, FL, true, W2, fail, H) :-
        writelist([I, F, tab, tab]), write_wrapper(Term), writelist(['  (', N, ')\n']),
        reportFoci(I, FL, W2, fail, H).

reportWrap(I, F, - N, Term, FL, fail, W2, true, H) :-
        write(I), write_wrapper(Term), nl,
        bar(I),
        atom_append('|  ', I, J),
           reportFoci(J, FL, W2, true, H),
        bar(I),
	writeln(I).

reportWrap(I, F, - N, Term, FL, true, W2, fail, H) :-
        write(I), write_wrapper(Term), nl,
       reportFoci(I, FL, W2, fail, H).




reportFoci(I, [], _, Pr, []).
reportFoci(I, [(F, initial_focus)], _, Pr, []) :-
	(Pr -> writelist([I, F, '\n']) ; true).

reportFoci(I, [(F, trans_dir(A, B, C))|FL], W, Pr, H) :-
        reportFoci(I, FL, W, Pr, H1),
        (Pr -> writelist([I, F, tab, tab, 'by trans(', A]),
	       (C = [] -> true ; writelist([', ', C])),
	       writeln(')')
	    ;  
	       true),
	append([A|C], H1, H).

reportFoci(I, [(F, wrap(WN, Term, Foci))|FL], W, Pr, H) :-
        reportFoci(I, FL, W, Pr, H1),
        reportWrap(I, F, WN, Term, Foci, W, Pr, H2), 
	append(H1, H2, H).


reportFoci(I, [(F, open_pos(WN, Term, Proof))|FL], W, Pr, H) :-
        reportFoci(I, FL, W, Pr, H1),
        check_open(WN, W, Pr2),
	(Pr2 -> writeln(I) ; 
		(Pr -> W2 = [] ; 
		       W2 = W)),
        reportProof(I, Proof, W2, Pr2, H2),
	(Pr2 -> writeln(I) ; true),
        (Pr -> writelist([I, F, tab, tab, Term, '  * ', WN, '\n']) ; true),
        append(H1, H2, H).


/*
** check_open(Nr, WrapperList, OldPrint, NewPrint)
*/

check_open(- N, W,  true) :- member(N, W), !.
check_open(+ N, W,  true) :- member(+ N, W), !.
check_open(N, W,  true) :- integer(N), member(N, W), !.
check_open(N, W,  fail).





delete_all(N, L, R) :-
	once(delete(N, L, L1)), 
	once((delete_all(N, L1, R) ; (R = L1))).

unique([], []).
unique([A|B], [A|C]) :-
	delete_all(A, [A|B], B1),
	unique(B1, C).

partition_hyps([], [], []).
partition_hyps([E|L], [E|IL], TL) :-
	integer(E), !,
	partition_hyps(L, IL, TL).
partition_hyps([E|L], IL, [E|TL]) :-
	partition_hyps(L, IL, TL).

bubble_hyps(N, inherited, H, [N|H]).
bubble_hyps(_, translate(N), H, [N|H]).
bubble_hyps(_, _, H, H).

reportHyps(_, [], _, _, _, []).
reportHyps('1.3', [(N, T, _, Exp, _)|List], Hyps, W, Pr, NHout) :-
	member(N, Hyps), !,
	bubble_hyps(N, Exp, Hout, NHout),
	check_open_hyp(Exp, W, Pr, NExp),
        (Pr -> writelist([I, N, ': ', T, tab, tab, NExp, '\n']) ; true),
        reportHyps(I, List, Hyps, W, Pr, Hout).
reportHyps(I, [(N, T, _, Exp)|List], Hyps, W, Pr, NHout) :-
	I \== '1.3',
	member(N, Hyps), !,
	bubble_hyps(N, Exp, Hout, NHout),
	check_open_hyp(Exp, W, Pr, NExp),
        (Pr -> writelist([I, N, ': ', T, tab, tab, NExp, '\n']) ; true),
        reportHyps(I, List, Hyps, W, Pr, Hout).
reportHyps(I,  [_|List], Hyps, W, Pr, Hout) :-
        reportHyps(I, List, Hyps, W, Pr, Hout).


check_open_hyp(_, _, fail, _) :- !.
check_open_hyp(wrap(+ N, Term, Exp), W, true, Term * N) :- \+ member(N, W), !.
check_open_hyp(wrap(N, Term, Exp), W, true, NE) :- !, check_open_hyp(Exp, W, true, NE).
check_open_hyp(Term, W, true, Term).


reportTheorems(T) :-
	unique(T, T1),
	writelist(['Depends upon : ', T1, '\n\n\n']).

bar(I) :-
        write(I),
        atom_chars(I, L),
        length(L, N),
        M is 80 - N,
        bar_c(M).

bar_c(M) :-
        N is M - 3,
        N > 0, !,
        write('---'),
        bar_c(N).
bar_c(_) :- nl.

