%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS conj.ql,v 1.8 1993/12/10 02:18:36 marku Exp



/*
** Examine each of the conjuncts in an n-ary conjunction
**
** This is one of the (hopefully) few tactics which have to have anything
** to do with the wrapper mechanism.
*/

:- tactic(conjuncts(_), 
	  ['conjuncts([?Conj1,?Conj2,...]) -- split focus into conjuncts.'],
	  ['Splitting into conjuncts'],
	  [['Conjunct ', 1] | _]
         ).

conjuncts(L) :-
	conjuncts(L, _).

conjuncts([First, Next | Rest], Status) :-
	nonvar(First),
	is_focus(_ and _),
	First = call_tac(- N, Descr, _), 
	Next = call_tac(- N, NDescr, _),
	var(NDescr), 
	!,
	next_label(Descr, NDescr),
	do_conjuncts([First, Next | Rest], Status).

conjuncts(L, Status) :-
	do_conjuncts(L, Status).
	


do_conjuncts([Code | Rest], Status) :-
	is_focus(_ and _),
	!,
	open_pos([1], do_one_conjunct(Code,Status)),
	((is_focus(true and _) ->
	    trans(r:trueand),
	    Status = success,
	    (conjuncts(Rest, Status) ; true))
        ;
	    ((open_pos([2], conjuncts(Rest,Status)) ; true),
	     ((is_focus(_ and true) ->
		 trans(r:andtrue),
		 Status = success)
	     ;
	         true
	     ))
	),
	!,
	%% fail if we haven't changed anything.
	nonvar(Status).
		 
do_conjuncts([Code], Status) :-
	do_one_conjunct(Code, Status),
	nonvar(Status).


do_one_conjunct(Code, Status) :-
	is_focus(F0),
	focus([],Code),
	is_focus(F),
	((F == F0 ->
	    less_verbose(msg(['conjunct: ',F0,'  is unchanged.',nl])))
	;
	    (Status = success,
	     less_verbose(msg(['conjunct: ',F0,nl,'     ---> ',F,nl]))
	    )
	).



/*
** init_var(Arg, Default)
*/

init_var(A, B) :- init_var(A, B, A).

/*
** init_var(Arg, Suggested, Resultant)
*/

init_var(A, S, A) :- nonvar(A), !.
init_var(A, S, A) :- frozen(A), !.
init_var(A, S, S).

/*
** Guess at the next in sequence
*/

next_label(L1, L2) :-
	append(A, [N|B], L1),
	integer(N), !,
	N1 is N + 1,
	append(A, [N1|B], L2).

next_label(L, L).


/*
** Shorthand open_pos
*/

open_pos(A, B) :- open_pos(A, A, [], B).


