%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS int_simp.ql,v 1.3 1993/12/10 02:18:46 marku Exp


/*
	int_simp
	========

If the argument is an integer expression,
this tactic converts products to sums using the
distributive law. It converts terms containing
factors of zero, to zero. It converts sums
with one term equal to zero using the identity
rule for addition. It also adds like terms
(including integers) when they appear adjacent
to one another. 

If the argument is an inequality that uses the 
le relation, all terms are shifted to the left-hand
side, which is then simplified.
*/

?- tactic(int_simp(Term),
	[],
	'A tactic to simplify integer expressions AND integer relations using the inequality le',
	[]).

/* *********************************************
	CODE FOR int_simp TACTIC
************************************************/

int_simp(A =< B):-
	focus([1],int_simp(A)),
	focus([2],int_simp(B)),
	is_focus(Term),
	eqn_simp(Term).

eqn_simp(N1 =< 0):-
	integer(N1),
	!,
	use(arith:eval_not_le,false,_),
	quit.

eqn_simp(T1 =< 0):-
	focus([1],
		 (is_focus(Term),
		 int_simp(Term))).


eqn_simp(T1 =< -(T2)):-
	use(arith:le_add_zero,T1 + T2 =< 0,_),
        focus([1], 
                (is_focus(Term),
		 int_simp(Term))).

eqn_simp(T1 =< T2):-
	focus([2],
		use(arith:neg_neg_id,-(-(T2)),_)),
	use(arith:le_add_zero,T1 + -(T2) =< 0,_),
	focus([1],
		 (is_focus(Term),
		 int_simp(Term))).

int_simp(T1 + 0):-
	focus([1],int_simp(T1)),
	is_focus(T + 0),
	use(arith:add_id_r,T,_).

int_simp(0 + T1):-
	focus([2],int_simp(T1)),
	is_focus(0 + T),
	use(arith:add_id_l,T,_).

int_simp((-(N1)) + (-(N2))):-
	integer(N1),
	integer(N2),
	!,
	Ans is -N1 + (-N2),
	use(arith:eval_eq,Ans,_).

int_simp((-(N1)) + N2):-
	integer(N1),
	integer(N2),
	!,
	Ans is -N1 + N2,
	use(arith:eval_eq,Ans,_).

int_simp(N1 + (-(N2))):-
        integer(N1),
	integer(N2),
	!,
	Ans is N1 + (-N2),
	use(arith:eval_eq,Ans,_).

int_simp(N1 + N2):-
        integer(N1),
	integer(N2),
	!,
	Ans is N1 + N2,
	use(arith:eval_eq,Ans,_).

int_simp((-(N1))*T1 + (-(N2))*T1):-
        integer(N1),
        integer(N2),
	!,
        use(arith:mul_dist_add_l,((-(N1)) + (-(N2)))*T1,_),
        focus([1],
                (Ans is -N1 + (-N2),
                 use(arith:eval_eq,Ans,_))).

int_simp((-(N1))*T1 + N2*T1):-
        integer(N1),
        integer(N2),
	!,
        use(arith:mul_dist_add_l,((-(N1)) + N2)*T1,_),
        focus([1],
                (Ans is -N1 + N2,
                 use(arith:eval_eq,Ans,_))).

int_simp(N1*T1 + (-(N2))*T1):-
        integer(N1),
        integer(N2),
	!,
        use(arith:mul_dist_add_l,(N1 + (-(N2)))*T1,_),
        focus([1],
                (Ans is N1 + (-N2),
                 use(arith:eval_eq,Ans,_))).

int_simp(N1*T1 + N2*T1):-
        integer(N1),
        integer(N2),
	!,
        use(arith:mul_dist_add_l,(N1 + N2)*T1,_),
        focus([1],
        (Ans is N1 + N2,
         use(arith:eval_eq,Ans,_))).

int_simp(T1 + T2):-
	focus([1],
		(is_focus(T1),
		 int_simp(T1),
		 if( or(is_focus(X1 + 0),or(is_focus(0 + Y1),is_focus(0*Z1))),
			(is_focus(T3),
			 int_simp(T3)) ))),
	focus([2],
		(is_focus(T2),
		 int_simp(T2),
		 if( or(is_focus(X2 + 0),or(is_focus(0 + Y2),is_focus(0*Z2))),
			(is_focus(T4),
			 int_simp(T4)) ))),
	if( or(is_focus(X3 + 0),or(is_focus(0 + Y3),is_focus(0*Z3))),
	       (is_focus(T5),
	       int_simp(T5)) ).

int_simp(-(-(T1))):-
	focus([1,1],int_simp(T1)),
	is_focus(-(-(T))),
	use(arith:neg_neg_id,T,_),
	is_focus(Term),
	int_simp(Term).

int_simp(0*T1):-
	use(arith:mul_zero_l,0,_).


int_simp(T1*0):-
	use(arith:mul_zero_r,0,_).

int_simp(F1*(T1 + T2)):-
	use(arith:mul_dist_add_r,
		F1*T1 + F1* T2, _),
	is_focus(Term),
	int_simp(Term).

int_simp(-(N1)*(-(N2)*T1)):-
	integer(N1),
	integer(N2),
	use(arith:mul_assoc,((-(N1))*(-(N2)))*T1,_),
	focus([1],
		(Ans is -N1*(-N2),
		 use(arith:eval_eq,Ans,_))),
	is_focus(Term),
	int_simp(Term).

int_simp(N1*(-(N2)*T1)):-
	integer(N1),
	integer(N2),
	use(arith:mul_assoc,(N1*(-(N2)))*T1,_),
	focus([1],
		(Ans is N1*N2,
		 use(arith:eval_eq,-(Ans),_))),
	is_focus(Term),
	int_simp(Term).

int_simp((-(N1))*(N2*T1)):-
	integer(N1),
	integer(N2),
	use(arith:mul_assoc,((-(N1))*N2)*T1,_),
	focus([1],
		(Ans is N1*N2,
		 use(arith:eval_eq,-(Ans),_))),
	is_focus(Term),
	int_simp(Term).
		 

int_simp(N1*(N2*T1)):-
	integer(N1),
	integer(N2),
	use(arith:mul_assoc,(N1*N2)*T1,_),
	focus([1],
		(Ans is N1*N2,
		 use(arith:eval_eq,Ans,_))),
	is_focus(Term),
	int_simp(Term).

int_simp(-(N1)*(-(N2))):-
	integer(N1),
	integer(N2),
	Ans is N1*N2,
	use(arith:eval_eq,Ans,_).

int_simp(-(N1)*N2):-
	integer(N1),
	integer(N2),
	Ans is N1*N2,
	use(arith:eval_eq,-(Ans),_).

int_simp(N1*(-(N2))):-
	integer(N1),
	integer(N2),
	Ans is N1*N2,
	use(arith:eval_eq,-(Ans),_).

int_simp(N1*N2):-
	integer(N1),
	integer(N2),
	Ans is N1*N2,
	use(arith:eval_eq,Ans,_).

int_simp(-(T1 + T2)):-
	use(arith:neg_dist_add,(-(T1)) + (-(T2)),_),
	is_focus(Term),
	int_simp(Term).

int_simp(-(F1 * F2)):-
	use(arith:neg_dist_mul_l,(-(F1))*F2,_),
	focus([1],
		(is_focus(T),int_simp(T))),
	is_focus(Term),
	int_simp(Term).

int_simp(_).



/*
	re_assoc_terms
	==============

This tactic flattens a sum to a left-associative 
structure.
*/


?- tactic(re_assoc_terms(Term),
	[],
	'A tactic to left-associate all terms in a sum',
	[]).

re_assoc_terms(T1 + (T2 + (T3 + T4))):-
	use(arith:add_assoc,(T1 + T2) + (T3 + T4),_),
	is_focus(Term),
	re_assoc_terms(Term).

re_assoc_terms(T1 + (T2 + T3)):-
	use(arith:add_assoc,(T1 + T2) + T3,_),
	focus([1],
		(is_focus(Term),
		 re_assoc_terms(Term))).


re_assoc_terms((T1 + T2) + T3):-
	focus([1],
		(is_focus(Term),
		 re_assoc_terms(Term))).

re_assoc_terms(T1 + T2).


/*
	sort_and_simp
	=============

This tactic sorts a sum of linear terms, adding 
like terms along the way.
*/

?- tactic(sort_and_simp(Term),
	[],
	'A tactic that sorts terms, adding like terms when they are adjacent',
	[]).

?- tactic(sort_and_simp_some(Term),
	[],
	'A tactic that sorts terms',
	[]).

sort_and_simp(Term):-
	nos_terms_in(Term,Nos),
	bubble_sort(1,Nos).

nos_terms_in(T1 + T2,Nos):-
	!,
	nos_terms_in(T1,N1),
	nos_terms_in(T2,N2),
	Nos is N1 + N2.

nos_terms_in(T1,1).

bubble_sort(Bott,Top):-
	not(Bott = Top),
	is_focus(Term),
	sort_and_simp_some(Term),
	Newtop is Top - 1,
	bubble_sort(Bott,Newtop).

bubble_sort(Bott,Bott).

sort_and_simp_some(T1 + 0):-
	use(arith:add_id_r,T1,_),
	is_focus(Term),
	sort_and_simp_some(Term).

sort_and_simp_some(0 + T1):-
	use(arith:add_id_l,T1,_),
	is_focus(Term),
	sort_and_simp_some(Term).
	
sort_and_simp_some((T1 + (-(N1))) + (-(N2))):-
	integer(N1),
	integer(N2),
	!,
	use(arith:add_assoc,T1 + ((-(N1)) + (-(N2))),_),
	focus([2],
		(Ans is (N1 + N2),
		 use(arith:eval_eq,-(Ans),_))),
	is_focus(Term),
	sort_and_simp_some(Term).


sort_and_simp_some((T1 + (-(N1))) + N2):-
	integer(N1),
	integer(N2),
	!,
	use(arith:add_assoc,T1 + ((-(N1)) + N2),_),
	focus([2],
		(if((N1 > N2),
			(Ans is (N1 - N2),
			 use(arith:eval_eq,-(Ans),_)),
			(Ans is (N2 - N1),
			 use(arith:eval_eq,Ans,_))))),
	is_focus(Term),
	sort_and_simp_some(Term).

sort_and_simp_some((T1 + N1) + (-(N2))):-
	integer(N1),
	integer(N2),
	use(arith:add_assoc,T1 + (N1 + (-(N2))),_),
	focus([2],
		(if((N2 > N1),
			(Ans is (N2 - N1),
			 use(arith:eval_eq,-(Ans),_)),
			(Ans is (N1 - N2),
			 write('got in correct if branch'),
			 nl,
			 use(arith:eval_eq,Ans,_))))),
	is_focus(Term),
	sort_and_simp_some(Term).

sort_and_simp_some((T1 + N1) + N2):-
	integer(N1),
	integer(N2),
	!,
	use(arith:add_assoc,(T1 + (N1 + N2)),_),
	focus([2],
		(Ans is (N1 + N2),
		 use(arith:eval_eq,Ans,_))),
        is_focus(Term),
	sort_and_simp_some(Term).

sort_and_simp_some((T1 + C1*T2) + C2*T2):-
	!,
	use(arith:add_assoc,(T1 + (C1*T2 + C2*T2)),_),
	focus([2],
		(use(arith:mul_dist_add_l,(C1 + C2)*T2,_),
		 focus([1],
			add_coeffs(C1 + C2)),
		 if(is_focus(0*_),
			use(arith:mul_zero_l,0,_)))),
	is_focus(Term),                 
	sort_and_simp_some(Term).

sort_and_simp_some((T1 + C1*T2) + C2*T3):-
	comes_first(T3,T2),
	use(arith:add3_sym,(T1 + C2*T3) + C1*T2,_),
	focus([1],
		(is_focus(Term),
		 sort_and_simp_some(Term))).
			

sort_and_simp_some((T1 + C1*T2) + C2*T3):-
	focus([1],
		(is_focus(Term),
		 sort_and_simp_some(Term))).


sort_and_simp_some((T1 + (-(N1))) + T2):-
	integer(N1),
	use(arith:add3_sym,(T1 + T2) + (-(N1)),_),
	focus([1],
		(is_focus(Term),
		 sort_and_simp_some(Term))).

sort_and_simp_some((T1 + N1) + T2):-
	integer(N1),
	use(arith:add3_sym,(T1 + T2) + N1,_),
	focus([1],
		(is_focus(Term),
		 sort_and_simp_some(Term))).

sort_and_simp_some((T1 + T2) + T3):-
	focus([1],
		(is_focus(Term),
		 sort_and_simp_some(Term))).

sort_and_simp_some(C1*T1 + C2*T1):-
	use(arith:mul_dist_add_l,(C1 + C2)*T1,_),
	focus([1],
		add_coeffs(C1 + C2)),
	if(is_focus(0*_),
		use(arith:mul_zero_l,0,_)).

sort_and_simp_some(C1*T1 + C2*T2):-
	comes_first(T2,T1),
	use(arith:add_sym,C2*T2 +  C1*T1,_).

sort_and_simp_some(C1*T1 + C2*T2).

sort_and_simp_some((-(N1)) + C1*T1):-
	integer(N1),
	use(arith:add_sym,C1*T1 + (-(N1)),_).

sort_and_simp_some(N1 + C1*T1):-
	integer(N1),
	use(arith:add_sym,C1*T1 + N1,_).

sort_and_simp_some(N1 + N2):-
	add_coeffs(N1 + N2).

sort_and_simp_some(_).

add_coeffs(-(N1) + (-(N2))):-
	integer(N1),
	integer(N2),
	Ans is N1+N2,
	use(arith:eval_eq,(-(Ans)),_).

add_coeffs(-(N1) + N2):-
	integer(N1),
	integer(N2),
	if(N1 > N2,
		(Ans is N1 - N2,
		 use(arith:eval_eq,(-(Ans)),_)),
		(Ans is N2 - N1,
		 use(arith:eval_eq,Ans,_))).

add_coeffs(N1 + (-(N2))):-
	integer(N1),
	integer(N2),
	if(N2 > N1,
		(Ans is N2 - N1,                
		 use(arith:eval_eq,(-(Ans)),_)),
		(Ans is N1 - N2,
		 use(arith:eval_eq,Ans,_))).

add_coeffs(N1 + N2):-
	integer(N1),
	integer(N2),
	Ans is N1+N2,
	use(arith:eval_eq,Ans,_).


comes_first(T1,T2):-
	is_hyp(N1,int(T1)),
	is_hyp(N2,int(T2)),
	N1 < N2.

/*
	add_like_terms
	==============

This tactic adds like terms that appear adjacent to
one another in a sum.
*/


?- tactic(add_like_terms(Term),
	[],
	'A tactic to add like terms',
	[]).

add_like_terms((T1 + C1*T2) + C2*T2):-
	use(arith:add_assoc,T1 + (C1*T2 + C2*T2),_),
	focus([2],
		(use(arith:mul_dist_add_l,(C1 + C2)*T2,_),
		 focus([1],
			(add_coeffs(C1 + C2))))),
	is_focus(Term),
	add_like_terms(Term).


add_like_terms((T1 + C1*T2) + T3):-
	focus([1],
		(add_like_terms(T1 + C1*T2))).


add_like_terms((T1 + N1) + N2):-
	use(arith:add_assoc,T1 + (N1 + N2),_),
	focus([2],
		(add_coeffs(N1 + N2))),
	is_focus(Term),
	add_like_terms(Term).

add_like_terms(C1*T1 + C2*T1):-
	use(arith:mul_dist_add_l,(C1 + C2)*T2,_),
	focus([1],
		(add_coeffs(C1 + C2))).

add_like_terms(N1 + N2):-
	add_coeffs(N1 + N2).


add_like_terms(_).


