%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS mark1.ql,v 1.11 1993/12/10 02:18:55 marku Exp


:- tactic(generalize(Term,ObjVar),
	 ['generalize(+Term,?Objvar) generalises the focus by replacing',nl,
	   'all occurrences of Term by Objvar and wrapping the resulting',nl,
	   'term within an ''all'' quantifier'],
	 ['Generalizing ',A,' with ',B],
	 []).

:- tactic(focus_on(Patt, _, _),
	['focus_on(+Pattern, -Path, ?SubProof) focuses on a subterm',nl,
	 'of the focus that matches Pattern'],
	['Focusing on ',Patt],
	['']).

:- tactic(focus_on(Patt, _),
	['focus_on(+Pattern, ?SubProof) focuses on a subterm',nl,
	 'of the focus that matches Pattern'],
	['Focusing on ',Patt],
	['']).

:- tactic(false(Query),
	['false(?Query) attempts to transform the focus F',nl,
	 'to false, by searching for a theorem matching (not F)'],
	['transform focus to false'],
	[]).

false(Query) :-
	is_focus(F),
	less_verbose(
	    once((
	        fwd(Query, not F),   %% only necessary sometimes!
	        trans_dir(r:notimpbool, f, [h (not F)])
	        ))
	).


/*
 * focus_on( Pattern, Path, Code )
 * ========================
 *
 *	Equivalent to focus(Path, Code), but it generates
 *	Path by searching for Pattern in the current focus.
 *
 * 	**** Make a copy of pattern to store in the proof record
 */

focus_on(Pattern, Code) :-
	focus_on(Pattern, _, Code).

focus_on(Pattern, Path, C) :-
	once(elem(Code, C)),
	is_focus(F),
	ex_any_path(F, Pattern, Path),
	focus(Path, Code).

/*
** Extract all subterms of a term
*/

ex_all_paths(Term, Term, []).

ex_all_paths(Term, Subterm, [Pos|Path]) :-
    Term =.. [O,A|Args],
    append([O|Pre], [B | _], [O,A|Args]),
    ex_all_paths(B, Subterm, Path),
    length([O|Pre], Pos).

/*
** Make various uses of the above
*/

ex_any_path(Term, Pattern, Pos) :-
	ex_all_paths(Term, Pattern, Pos),
	incomplete_retry_delays.

ex_any_thaw_path(Term, Pattern, Pos) :-
	thawing_new_vars(
		once (ex_all_paths(Term, Pattern, Pos),
		      incomplete_retry_delays)).

ex_path(T1, T2, P) :-
	once(ex_any_path(T1, T2, P)).

ex_thaw_path(T1, T2, P) :-
	once(ex_any_thaw_path(T1, T2, P)).


/*
 * generalize(MetaVar, ObjVar)
 *
 *	Uses the 'install' rule to generalize the focus by wrapping
 *	an all quantifier around it.  The bound variable of the resulting
 *	quantified term will be ObjVar and all occurences of MetaVar
 *	in the focus will be replaced by ObjVar.
 *
 *	Eg.  if the focus is: A + B = B + A
 *		then calling generalize(B,x,_) will convert the focus to
 *		(all x A + x = x + A) and will generate any necessary
 *		constraints about x not being free in A.
 */


generalize(X, x) :-
	is_focus(F),
	is_goal(true),   % could check is_reln(rev(=>)) instead?
	rev_sub(F, X, x, NTerm),
	trans(r:install, r, (all x NTerm), []).

	%	number_of_hyps(N),
	%       trans_goal(r:back_thm, if((all x NTerm), true, _), [r:install]),
	%       open_pos([1], (delete_new_hyps(N), Code)),
	%       trans(r:iftrue)


%% rev_sub(+Term, +Var, +ObjVar, +Term).
%%
%% Tempory: Var generalized to Term2

rev_sub(E,_,_,_) :-
	substitute(E, Subs, Term),
	\+ (Subs == []),
	!.

rev_sub(E,E,x,x) :- % was E,X,x,x
	%% var(E),
	%% E == X,
	!.

rev_sub(E,X,x,E) :-
	var(E),
	%% E \== X,
	!,
	x not_free_in E.

rev_sub(E,X,x,E) :-
	atomic(E),
	!.

rev_sub(E,X,x,E) :-
	is_object_var(E),
	!,
	x not_free_in E.

rev_sub(E,X,x,E2) :-
	E =.. [F|Args],
	!,
	rev_sublist(Args,X,x,Args2),
	E2 =.. [F|Args2].

rev_sub(E,X,x,E2) :-
	quantify(E,Q,V,B),
	quantify(E2,Q,V,B2),
	!,
	x not_free_in V,
	rev_sub(B,X,x,B2).


rev_sublist([],_,_,[]).

rev_sublist([A|As],X,x,[B|Bs]) :-
	rev_sub(A,X,x,B),
	!,
	rev_sublist(As,X,x,Bs).


/*
** Homeless
*/
%% A convenient predicate for finding paths to subterms of the focus.
is_focus_subterm(Term,Path) :-
    is_focus(F),
    ex_any_path(F,Term,Path).



/*
** Backwards compatibility - purge.purge.purge
*/

change_focus(F) :-
	writeln('WARNING: change_focus/1 has been replaced by set_focus/1'),
	set_focus(F).

is_equ(E) :-
	writeln('WARNING: is_equ/1 has been replaced by is_reln/1'),
	is_reln(E).


