%% ERGO COPYRIGHT NOTICE, LICENCE AND DISCLAIMER.
%%
%% Copyright 1993 by The University of Queensland, Queensland 4072 Australia
%% 
%% 
%% Permission to use, copy and distribute this software for any
%% non-commercial purpose and without fee is hereby granted, provided
%% that the above copyright notice and this permission notice and
%% warranty disclaimer appear in all copies and in supporting
%% documentation, and that the name of the University of Queensland not
%% be used in advertising or publicity pertaining to distribution of
%% the software without specific, written prior permission.
%% 
%% Source code modifications are prohibited except where written 
%% agreement has been given in advance by the University of Queensland.
%% 
%% The University of Queensland disclaims all warranties with regard to
%% this software, including all implied warranties of merchantability
%% and fitness.  In no event shall the University of Queensland be
%% liable for any special, indirect or consequential damages or any
%% damages whatsoever resulting from loss of use, data or profits,
%% whether in an action of contract, negligence or other tortious
%% action, arising out of or in connection with the use or performance
%% of this software.
%% 
%% CVS quick.ql,v 1.6 1993/12/10 02:19:02 marku Exp


/*
** Absolutely inflexible rerun
*/

:- declare(q_rerun/1).


q_rerun(Thm) :-
	once((decode_rule(Thm, CD, Name), 
	      atom(Name),
	      cd(CD),
	      get_proof_record(CD:Name, proof_of(_, Form, Ver, Proof)),
	      writeln(CD:Name))),
	q_ver_msg(Ver),
	theorem(CD:Name, Form, once(q_rerun_theorem(Proof))).

q_ver_msg('1.2') :- !.
q_ver_msg(B) :-
	writelist(['Rerun of version ', B, ' proof records not supported by q_rerun/1\n']), !,
	fail.

q_rerun_theorem(P) :-
	ipSet(abandon_proof_record, very_true),
	ipSet(new_proof, false),
	q_rerun_proof(P).

q_rerun_proof((_, FL, Hyps)) :-
	q_rerun_hyps(Hyps),
	q_rerun_foci(FL).


q_rerun_foci([(F, initial_focus)]).
q_rerun_foci([(F, trans_dir(A, B, C))|FL]) :-
        q_rerun_foci(FL),
	trans(A, B, F, C).
q_rerun_foci([(F, wrap(_, _, Foci))|FL]) :-
        q_rerun_foci(FL),
        q_rerun_foci(Foci). 
q_rerun_foci([(F, open_pos(_, Pos, Proof))|FL]) :-
        q_rerun_foci(FL),
        open_pos(Pos, Pos, [], q_rerun_proof(Proof)).


q_rerun_hyps([]).
q_rerun_hyps([(N, T, _, wrap(_, _, Exp))|List]) :-
	q_rerun_hyps([(N, T, _, Exp)|List]).
q_rerun_hyps([(N, T, _, take_theorem(R))|List]) :-
	q_rerun_hyps(List),
	take_theorem(R).
q_rerun_hyps([(N, T, _, h_trans_dir(R, A, B, C))|List]) :-
	q_rerun_hyps(List),
	h_trans(R, A, B, _, C).
q_rerun_hyps([(N, T, _, split_hyp(Nr))|List]) :-
        q_rerun_hyps(List),
	(is_hyp(Nr, _) -> split_hyp(Nr) ; true).
q_rerun_hyps([(N, T, _, open_hyp(_, Nr, Proof))|List]) :-
        q_rerun_hyps(List),
	open_hyp(Nr, q_rerun_proof(Proof)).
q_rerun_hyps([(N, T, _, inherited)|List]) :-
	q_rerun_hyps(List).
q_rerun_hyps([(N, T, _, translated(_))|List]) :-
        q_rerun_hyps(List).
q_rerun_hyps([(N, T, _, 'introduced')|List]) :-
        q_rerun_hyps(List).

